﻿########################################################################
# Created On: 05/05/2013 00:15
# Created By: Thibault
# Script: Manage your checkpoints on Windows in PowerShell
# More: This script is provided as is without warranty of any kind.
#		Free for use/modify as you want (but at your own risk).
#		It's just training :)
########################################################################

function MakeSomeMagic {

########################################################################
# Assemblies calling
########################################################################

#region Import ssemblies
[reflection.assembly]::loadwithpartialname("System.Drawing") | Out-Null
[reflection.assembly]::loadwithpartialname("System.Windows.Forms") | Out-Null
#endregion

########################################################################
# Form objects Declaration
########################################################################

#region Form Objects Declaration
$form1 = New-Object System.Windows.Forms.Form
$panel1 = New-Object System.Windows.Forms.Panel
$button_RestoreComputer = New-Object System.Windows.Forms.Button
$groupBox_LastRP = New-Object System.Windows.Forms.GroupBox
$label_lastRP = New-Object System.Windows.Forms.Label
$button_Quit = New-Object System.Windows.Forms.Button
$button_CreateRestorePoint = New-Object System.Windows.Forms.Button
$groupBox_ListRestorePoints = New-Object System.Windows.Forms.GroupBox
$listbox = New-Object System.Windows.Forms.ListBox
$groupBox_Details = New-Object System.Windows.Forms.GroupBox
$label_Details = New-Object System.Windows.Forms.Label
$InitialFormWindowState = New-Object System.Windows.Forms.FormWindowState
#endregion Form Objects Declaration

########################################################################
# Manage all the actions of the components
########################################################################

#region Actions of the Windows Form (Button etc.)
$button_CreateRestorePoint_OnClick= 
{
$created_date_checkpoint = Get-date -Format %d-%M-%y
Checkpoint-Computer -Description "Checkpoint automatically generated in Powershell on $created_date_checkpoint"
$button_CreateRestorePoint.Enabled = $false
$button_CreateRestorePoint.Text = "Checkpoint created"
}

$button_Quit_OnClick= 
{
$form1.Close();
}

$button_RestoreComputer_OnClick= 
{
#Will restore the computer with the last checkpoint
Restore-Computer -RestorePoint $id
}

#Correct the initial state of the form to prevent the .Net maximized form issue
$OnLoadForm_StateCorrection=
{
	$form1.WindowState = $InitialFormWindowState
    $form1.FormBorderStyle = [System.Windows.Forms.FormBorderStyle]::FixedDialog #Fixed dialog
    $form1.MaximizeBox = $False #No maximize button
    $form1.MinimizeBox = $False #No minimize button

    #To avoid the mistakes ;)
    #If your launch the script with elevated permissions there will be no question or confirmation
    #The restoration process will start... So, you are warn - Use at your own risk. 
    $button_RestoreComputer.Enabled = $false 
}
#endregion Actions of the Windows Form (Button etc.)

########################################################################
# Configure all the components of the form
########################################################################

#region Form Code
#Form configuration
$System_Drawing_Size = New-Object System.Drawing.Size
$System_Drawing_Size.Height = 472
$System_Drawing_Size.Width = 489
$form1.ClientSize = $System_Drawing_Size
$form1.DataBindings.DefaultDataSourceUpdateMode = 0
$form1.FormBorderStyle = 3
$form1.Name = "form1"
$form1.Text = "Managing your restore points"
$form1.add_Load($handler_form1_Load)

#Panel1 configuration
$panel1.DataBindings.DefaultDataSourceUpdateMode = 0
$System_Drawing_Point = New-Object System.Drawing.Point
$System_Drawing_Point.X = 12
$System_Drawing_Point.Y = 12
$panel1.Location = $System_Drawing_Point
$panel1.Name = "panel1"
$System_Drawing_Size = New-Object System.Drawing.Size
$System_Drawing_Size.Height = 448
$System_Drawing_Size.Width = 468
$panel1.Size = $System_Drawing_Size
$panel1.TabIndex = 1

$form1.Controls.Add($panel1)

#Button RestoreComputer configuration
$button_RestoreComputer.DataBindings.DefaultDataSourceUpdateMode = 0
$System_Drawing_Point = New-Object System.Drawing.Point
$System_Drawing_Point.X = 160
$System_Drawing_Point.Y = 384
$button_RestoreComputer.Location = $System_Drawing_Point
$button_RestoreComputer.Name = "button_RestoreComputer"
$System_Drawing_Size = New-Object System.Drawing.Size
$System_Drawing_Size.Height = 52
$System_Drawing_Size.Width = 145
$button_RestoreComputer.Size = $System_Drawing_Size
$button_RestoreComputer.TabIndex = 5
$button_RestoreComputer.Text = "Restore your computer with the LAST checkpoint"
$button_RestoreComputer.UseVisualStyleBackColor = $True
$button_RestoreComputer.add_Click($button_RestoreComputer_OnClick)

$panel1.Controls.Add($button_RestoreComputer)

#GroupBox LastRP
$groupBox_LastRP.DataBindings.DefaultDataSourceUpdateMode = 0
$System_Drawing_Point = New-Object System.Drawing.Point
$System_Drawing_Point.X = 9
$System_Drawing_Point.Y = 98
$groupBox_LastRP.Location = $System_Drawing_Point
$groupBox_LastRP.Name = "groupBox_LastRP"
$System_Drawing_Size = New-Object System.Drawing.Size
$System_Drawing_Size.Height = 61
$System_Drawing_Size.Width = 447
$groupBox_LastRP.Size = $System_Drawing_Size
$groupBox_LastRP.TabIndex = 4
$groupBox_LastRP.TabStop = $False
$groupBox_LastRP.Text = "Last restore point date"

$panel1.Controls.Add($groupBox_LastRP)
$label_lastRP.DataBindings.DefaultDataSourceUpdateMode = 0
$label_lastRP.Font = New-Object System.Drawing.Font("Microsoft Sans Serif",8.25,1,3,0)
$label_lastRP.ForeColor = [System.Drawing.Color]::FromArgb(255,0,0,0)

#Label LastRP configuration
$System_Drawing_Point = New-Object System.Drawing.Point
$System_Drawing_Point.X = 6
$System_Drawing_Point.Y = 24
$label_lastRP.Location = $System_Drawing_Point
$label_lastRP.Name = "label_lastRP"
$System_Drawing_Size = New-Object System.Drawing.Size
$System_Drawing_Size.Height = 23
$System_Drawing_Size.Width = 431
$label_lastRP.Size = $System_Drawing_Size
$label_lastRP.TabIndex = 0
#region Display data about the last Restore Point
#Get the date of the Last Restore Point
$var = Get-ComputerRestorePoint | Sort-Object SequenceNumber | Select-Object @{Label="Date"; Expression={$_.ConvertToDateTime($_.CreationTime)}} , @{Label="Age"; Expression={(((Get-Date) - $_.ConvertToDateTime($_.CreationTime))).Days}}, Description, SequenceNumber -Last 1
$id = $var.SequenceNumber
$date = $var.Date
$age = $var.Age
$label_lastRP.Text = "You last restore point is $age day(s) old ($date) with the ID : $id."
#endregion
$groupBox_LastRP.Controls.Add($label_lastRP)

#Button Quit configuration
$button_Quit.DataBindings.DefaultDataSourceUpdateMode = 0
$System_Drawing_Point = New-Object System.Drawing.Point
$System_Drawing_Point.X = 311
$System_Drawing_Point.Y = 384
$button_Quit.Location = $System_Drawing_Point
$button_Quit.Name = "button_Quit"
$System_Drawing_Size = New-Object System.Drawing.Size
$System_Drawing_Size.Height = 52
$System_Drawing_Size.Width = 145
$button_Quit.Size = $System_Drawing_Size
$button_Quit.TabIndex = 3
$button_Quit.Text = "Quit"
$button_Quit.UseVisualStyleBackColor = $True
$button_Quit.add_Click($button_Quit_OnClick)

$panel1.Controls.Add($button_Quit)

#Button CreateRestorePoint
$button_CreateRestorePoint.DataBindings.DefaultDataSourceUpdateMode = 0
$System_Drawing_Point = New-Object System.Drawing.Point
$System_Drawing_Point.X = 9
$System_Drawing_Point.Y = 384
$button_CreateRestorePoint.Location = $System_Drawing_Point
$button_CreateRestorePoint.Name = "button_CreateRestorePoint"
$System_Drawing_Size = New-Object System.Drawing.Size
$System_Drawing_Size.Height = 52
$System_Drawing_Size.Width = 145
$button_CreateRestorePoint.Size = $System_Drawing_Size
$button_CreateRestorePoint.TabIndex = 2
$button_CreateRestorePoint.Text = "Create a restore point"
$button_CreateRestorePoint.UseVisualStyleBackColor = $True
$button_CreateRestorePoint.add_Click($button_CreateRestorePoint_OnClick)

$panel1.Controls.Add($button_CreateRestorePoint)

#GroupBox ListRestorePoints
$groupBox_ListRestorePoints.DataBindings.DefaultDataSourceUpdateMode = 0
$System_Drawing_Point = New-Object System.Drawing.Point
$System_Drawing_Point.X = 9
$System_Drawing_Point.Y = 165
$groupBox_ListRestorePoints.Location = $System_Drawing_Point
$groupBox_ListRestorePoints.Name = "groupBox_ListRestorePoints"
$System_Drawing_Size = New-Object System.Drawing.Size
$System_Drawing_Size.Height = 213
$System_Drawing_Size.Width = 447
$groupBox_ListRestorePoints.Size = $System_Drawing_Size
$groupBox_ListRestorePoints.TabIndex = 1
$groupBox_ListRestorePoints.TabStop = $False
$groupBox_ListRestorePoints.Text = "Restore points list"

$panel1.Controls.Add($groupBox_ListRestorePoints)

#ListBox that contains all the last RestoreComputer points
$listbox.DataBindings.DefaultDataSourceUpdateMode = 0
$System_Drawing_Point = New-Object System.Drawing.Point
$System_Drawing_Point.X = 6
$System_Drawing_Point.Y = 19
$listbox.Location = $System_Drawing_Point
$listbox.Name = "listbox1"
$System_Drawing_Size = New-Object System.Drawing.Size
$System_Drawing_Size.Height = 188
$System_Drawing_Size.Width = 432
$listbox.Size = $System_Drawing_Size
$listbox.TabIndex = 0

#GroupBox details
$groupBox_Details.DataBindings.DefaultDataSourceUpdateMode = 0
$System_Drawing_Point = New-Object System.Drawing.Point
$System_Drawing_Point.X = 9
$System_Drawing_Point.Y = 3
$groupBox_Details.Location = $System_Drawing_Point
$groupBox_Details.Name = "groupBox_Details"
$System_Drawing_Size = New-Object System.Drawing.Size
$System_Drawing_Size.Height = 88
$System_Drawing_Size.Width = 447
$groupBox_Details.Size = $System_Drawing_Size
$groupBox_Details.TabIndex = 0
$groupBox_Details.TabStop = $False
$groupBox_Details.Text = "Details"

$panel1.Controls.Add($groupBox_Details)
$label_Details.DataBindings.DefaultDataSourceUpdateMode = 0

#Label "Bla-Bla explanations"
$System_Drawing_Point = New-Object System.Drawing.Point
$System_Drawing_Point.X = 7
$System_Drawing_Point.Y = 20
$label_Details.Location = $System_Drawing_Point
$label_Details.Name = "label_Details"
$System_Drawing_Size = New-Object System.Drawing.Size
$System_Drawing_Size.Height = 54
$System_Drawing_Size.Width = 431
$label_Details.Size = $System_Drawing_Size
$label_Details.TabIndex = 0
$label_Details.Text = "Here, you can check the last restore points that have been created on your computer. It can be personal restore points that you manually created.  Or it can be automatic restore points created by Windows Updates or installation softwares. For each point, you will get the date, age, description and the sequence number."

$groupBox_Details.Controls.Add($label_Details)
#endregion Generated Form Code

########################################################################
# Filling the ListBox in PowerShell
########################################################################
#region Fill the ListView1 with content
$listbox = new-object System.Windows.Forms.ListBox
#Fill the listbox with content
$listRP = Get-ComputerRestorePoint | Sort-Object SequenceNumber -Descending | Select-Object SequenceNumber, @{Label="Date"; Expression={$_.ConvertToDateTime($_.CreationTime)}} , @{Label="Age"; Expression={(((Get-Date) - $_.ConvertToDateTime($_.CreationTime))).Days}}, Description
foreach ($element in $listRP) {
$test = "ID : " + $element.SequenceNumber.ToString() + " - Date : " + $element.Date.ToString() + " - Age : " + $element.Age.ToString() + " - Desc : " + $element.Description.ToString()
[void]$listbox.Items.Add($test)
}
$listbox.Location = new-object System.Drawing.Size(10,20)   
$listbox.size = new-object System.Drawing.Size(425,190)
$groupBox_ListRestorePoints.Controls.Add($listbox)
#endregion Fill the ListView1 with content

#Save the initial state of the form
$InitialFormWindowState = $form1.WindowState

#Init the OnLoad event to correct the initial state of the form
$form1.add_Load($OnLoadForm_StateCorrection)

#Show the Form
$form1.ShowDialog()| Out-Null

} #End Function

cls

#Call the Function
MakeSomeMagic

########################################################################
# END OF SCRIPT
########################################################################